/*
** Copyright (C) 2003 Andrew Lockhart <andrew@ingsoc.net>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* $Id: wifi_datatypes.h,v 1.3 2003/07/27 03:22:30 andrew Exp $ */

/* Data types for wifi plugins */

#ifndef __WIFI_DATATYPES_H__
#define __WIFI_DATATYPES_H__

typedef enum {                  /* 802.11 frame types */
  TYPE_INVALID = -1,
  TYPE_MANAGEMENT = 0x00,
  TYPE_CONTROL = 0x01,
  TYPE_DATA = 0x02
} wifi_type_t;

typedef enum {                  /* 802.11 management frame subtypes */
  STYPE_INVALID = -1, 
  STYPE_ASSOCREQ = 0x00,
  STYPE_ASSOCRESP = 0x01,
  STYPE_REASSOC_REQ = 0x02,
  STYPE_REASSOC_RESP = 0x03,
  STYPE_PROBEREQ = 0x04,
  STYPE_PROBERESP = 0x05,
  STYPE_BEACON = 0x08,
  STYPE_ATIM = 0x09,
  STYPE_DISASSOC = 0x0a,
  STYPE_AUTH = 0x0b,
  STYPE_DEAUTH = 0x0c,

/* 802.11 control frame subtypes */
  STYPE_PS = 0x0a,
  STYPE_RTS = 0x0b,
  STYPE_CTS =  0x0c,
  STYPE_ACK = 0x0d,
  STYPE_CFEND = 0x0e,
  STYPE_CFEND_CFACK = 0x0f,

/* 802.11 data frame subtypes */
  STYPE_DATA = 0x00,
  STYPE_CFACK = 0x01,
  STYPE_CFPOLL = 0x02,
  STYPE_CFACK_CFPOLL = 0x03,
  STYPE_NULL = 0x04,
  STYPE_CFACK_NULL = 0x05,
  STYPE_CFPOLL_NULL = 0x06,
  STYPE_CFACK_CFPOLL_NULL = 0x07
} wifi_stype_t;

typedef struct _WifiElementID{		  /* 802.11 element id */
  u_int8_t id;
  u_int8_t len;
} WifiElementID;

typedef struct _WifiCapInfo{		  /* 802.11 capability information */
  u_int16_t cap_ess:1;
  u_int16_t cap_ibss:1;
  u_int16_t cap_cfpoll:1;
  u_int16_t cap_cfpoll_req:1;
  u_int16_t cap_wep:1;
  u_int16_t cap_reserved:11;
} WifiCapInfo;

#define _OFFSET_MANAGEMENT_BODY sizeof(WifiHdr) - 6

typedef struct _WifiBeacon{	  /* 802.11 beacon body */
#define _OFFSET_TIMESTAMP 0x00
  u_int32_t *timestamp1;
  u_int32_t *timestamp2;
#define _OFFSET_BEACON_INT _OFFSET_TIMESTAMP + 2 * sizeof(u_int32_t)
  u_int16_t *beacon_int;
#define _OFFSET_BEACON_CAP_INFO _OFFSET_BEACON_INT + sizeof(u_int16_t)
  WifiCapInfo *cap_info;  
#define _OFFSET_BEACON_SSID_ID _OFFSET_BEACON_CAP_INFO + sizeof(WifiCapInfo)
  WifiElementID *ssid_id;
#define _OFFSET_BEACON_SSID _OFFSET_BEACON_SSID_ID + sizeof(WifiElementID)
  u_int8_t *ssid;
  WifiElementID *rates_id;
  u_int8_t *rates;
  u_int8_t *current_chan;
  /* we don't care about the rest right now */
} WifiBeacon;


typedef struct _WifiAssocReq{	  /* 802.11 association request body */
#define _OFFSET_ASSOCREQ_CAP_INFO 0x00
  WifiCapInfo *cap_info;
#define _OFFSET_ASSOCREQ_LISTEN_INT _OFFSET_ASSOCREQ_CAP_INFO + sizeof(WifiCapInfo)
  u_int16_t *listen_int;
#define _OFFSET_ASSOCREQ_SSID_ID _OFFSET_ASSOCREQ_LISTEN_INT + sizeof(u_int16_t)
  WifiElementID *ssid_id;
#define _OFFSET_ASSOCREQ_SSID _OFFSET_ASSOCREQ_SSID_ID + sizeof(WifiElementID)
  u_int8_t *ssid;
  WifiElementID *rates_id;
  u_int8_t *rates;
} WifiAssocReq;

typedef struct _WifiReAssocReq{ /* 802.11 re-association request body */
#define _OFFSET_REASSOC_REQ 0x00
  WifiCapInfo *cap_info;
#define _OFFSET_REASSOC_REQ_LISTEN_INT _OFFSET_REASSOC_REQ + sizeof(WifiCapInfo)
  u_int16_t *listen_int;
#define _OFFSET_REASSOC_REQ_CURRENT_AP _OFFSET_REASSOC_REQ_LISTEN_INT + sizeof(u_int16_t)
  u_int8_t *current_ap;
#define _OFFSET_REASSOC_REQ_SSID_ID _OFFSET_REASSOC_REQ_CURRENT_AP + 6
  WifiElementID *ssid_id;
#define  _OFFSET_REASSOC_REQ_SSID _OFFSET_REASSOC_REQ_SSID_ID + sizeof(WifiElementID)
  u_int8_t *ssid;
  WifiElementID *rates_id;
  u_int8_t *rates;
} WifiReAssocReq;


typedef struct _WifiProbeReq{		  /* 802.11 probe request body */
#define _OFFSET_PROBEREQ_SSID_ID 0
  WifiElementID *ssid_id;
#define _OFFSET_PROBEREQ_SSID _OFFSET_PROBEREQ_SSID_ID + sizeof(WifiElementID)
  u_int8_t *ssid;
  WifiElementID *rates_id;
  u_int8_t *rates;
} WifiProbeReq;


typedef struct _WifiProbeResp{  /* 802.11 probe response body */
#define _OFFSET_TIMESTAMP 0x00
  u_int32_t *timestamp1;
  u_int32_t *timestamp2;
#define _OFFSET_BEACON_INT _OFFSET_TIMESTAMP + 2 * sizeof(u_int32_t)
  u_int16_t *beacon_int;
#define _OFFSET_PROBERESP_CAP_INFO _OFFSET_BEACON_INT + sizeof(u_int16_t)
  WifiCapInfo *cap_info;  
#define _OFFSET_PROBERESP_SSID_ID _OFFSET_PROBERESP_CAP_INFO + sizeof(WifiCapInfo)
  WifiElementID *ssid_id;
#define _OFFSET_PROBERESP_SSID _OFFSET_PROBERESP_SSID_ID + sizeof(WifiElementID)
  u_int8_t *ssid;
  WifiElementID *rates_id;
  u_int8_t *rates;
  u_int8_t *current_chan;
  /* we don't care about the rest right now */
} WifiProbeResp;





#endif /* __WIFI_DATATYPES_H__ */
