/*
** Copyright (C) 2003 Andrew Lockhart <andrew@ingsoc.net>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* $Id: spp_antistumbler.h,v 1.2 2003/07/24 06:32:37 andrew Exp $ */

#include "snort.h"

#ifndef __SPP_ANTISTUMBLER_H__
#define __SPP_ANTISTUMBLER_H__

#ifdef __SPP_ANTISTUMBLER_C__

#define ANTISTUMBLER_PROBE_REQS 5
#define ANTISTUMBLER_PROBE_PERIOD 3
#define ANTISTUMBLER_EXPIRE_TIMEOUT 60

typedef struct _Stumbler {
  u_int8_t addr[6];
  int alert_flag;
  int probe_cnt;
  time_t ts_init, ts_last;
  struct _Stumbler *next, *prev;
} Stumbler;

typedef struct _StumblerList {
  struct _Stumbler *head, *tail;
} StumblerList;

typedef struct _AntiStumblerData
{
  int probe_reqs;
  time_t probe_period;
  time_t expire_timeout;
  StumblerList *stumblers;
} AntiStumblerData;


typedef enum {                  /* 802.11 frame types */
  TYPE_INVALID = -1,
  TYPE_MANAGEMENT = 0x00,
  TYPE_CONTROL = 0x01,
  TYPE_DATA = 0x02
} wifi_type_t;


typedef enum {                  /* 802.11 management frame subtypes */
  STYPE_ASSOCREQ = 0x00,		  /* Subtypes are already AND'ed with their corresponding Types */
  STYPE_ASSOCRESP = 0x01,		  /* to prevent false positives and the need for checking the type */
  STYPE_REASSOC_REQ = 0x02,	  /* independently */
  STYPE_REASSOC_RESP = 0x03,
  STYPE_PROBEREQ = 0x04,
  STYPE_PROBERESP = 0x05,
  STYPE_BEACON = 0x08,
  STYPE_ATIM = 0x09,
  STYPE_DIASSOC = 0x0a,
  STYPE_AUTH = 0x0b,
  STYPE_DEAUTH = 0x0c,

  STYPE_INVALID = 0x0d,

/* 802.11 control frame subtypes */
  STYPE_PS = 0x0a,
  STYPE_RTS = 0x0b,
  STYPE_CTS =  0x0c,
  STYPE_ACK = 0x0d,
  STYPE_CFEND = 0x0e,
  STYPE_CFEND_CFACK = 0x0f,

/* 802.11 data frame subtypes */
  STYPE_DATA = 0x00,
  STYPE_CFACK = 0x01,
  STYPE_CFPOLL = 0x02,
  STYPE_CFACK_CFPOLL = 0x03,
  STYPE_NULLFUNC = 0x04,
  STYPE_CFACK_NULLFUNC = 0x05,
  STYPE_CFPOLL_NULLFUNC = 0x06,
  STYPE_CFACK_CFPOLL_NULLFUNC = 0x07
} wifi_stype_t;

typedef struct _ElementID{		  /* 802.11 element id */
  u_int8_t id;
  u_int8_t len;
} ElementID;

typedef struct _CapInfo{		  /* 802.11 capability information */
  u_int16_t cap_ess:1;
  u_int16_t cap_ibss:1;
  u_int16_t cap_cfpoll:1;
  u_int16_t cap_cfpoll_req:1;
  u_int16_t cap_wep:1;
  u_int16_t cap_reserved:11;
} CapInfo;

#define _OFFSET_MANAGEMENT_BODY sizeof(WifiHdr) - 6

typedef struct _ProbeReq{		  /* 802.11 probe request body */
#define _OFFSET_PROBEREQ_SSID_ID 0
  ElementID *ssid_id;
#define _OFFSET_PROBEREQ_SSID _OFFSET_PROBEREQ_SSID_ID + sizeof(ElementID)
  u_int8_t *ssid;
  ElementID *rates_id;
  u_int8_t *rates;
} ProbeReq;




typedef struct _Beacon{			  /* relevant portion of beacon frame 
											  for detecting Orinoco cards in monitor mode */
#define _OFFSET_TIMESTAMP 0x00
  u_int32_t *timestamp1;
  u_int32_t *timestamp2;

#define _OFFSET_BEACON_INT _OFFSET_TIMESTAMP + 2 * sizeof(u_int32_t)
  u_int16_t *beacon_interval;
  
#define _OFFSET_BEACON_CAP_INFO _OFFSET_BEACON_INT + sizeof(u_int16_t)
  CapInfo *cap_info;
  
#define _OFFSET_BEACON_SSID_ID _OFFSET_BEACON_CAP_INFO + sizeof(CapInfo)
  ElementID *ssid_id;
#define _OFFSET_BEACON_SSID _OFFSET_BEACON_SSID_ID + sizeof(ElementID)
  u_int8_t *ssid;

  ElementID *rates_id;
  u_int8_t *rates;
  
  u_int8_t *current_chan;
  
  /* we don't care about the rest right now */
} Beacon;

#define DECODE_PARAMS

StumblerList *AllocStumblerList(void);
void DestroyStumblerList(StumblerList *);
Stumbler *InsertStumbler(u_int8_t *, StumblerList *);
Stumbler *RemoveStumbler(Stumbler *, StumblerList *);

#endif /* __SPP_ANTISBUMBLER_C__ */

/* list of function prototypes for this preprocessor */
void SetupAntiStumbler();
void AntiStumblerInit(u_char *);
void AntiStumblerParseArgs(char *);
void AntiStumblerDetect(Packet *);
void PreprocRestartFunction(int);
void PreprocCleanExitFunction(int);


#endif  /* __SPP_ANTISTUMBLER_H__ */
