/*
** Copyright (C) 2004 sebastien Gracia <wireless@sgdn.pm.gouv.fr>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**
** Copyright disclaimers:
** The Directeur Central de la scurit des systmes d'information hereby
** disclaims all copyright interest in the changes and enhancements
** made by sebastien Gracia.
** 5th Fev 2004. 
*/


#ifndef __SPP_DEAUTH_FLOOD_H__
#define __SPP_DEAUTH_FLOOD_H__


/*  I N C L U D E S
*******************************************************************************/
#include "snort.h"


/*  D E F I N E S
*******************************************************************************/

#define DEAUTH_OPT_threshold "deauth-threshold"
#define DEAUTH_OPT_EXPIRE_TIMEOUT "expire_timeout"
#define DEAUTH_OPT_TARGET_LIMIT "target_limit"
#define DEAUTH_OPT_PRUNE_PERIOD "prune_period"


#define DEAUTH_FLOOD_THRESHOLD 20
#define DEAUTH_FLOOD_EXPIRE_TIMEOUT 60
#define DEAUTH_FLOOD_TARGET_LIMIT 100
#define DEAUTH_FLOOD_PRUNE_PERIOD 30

typedef struct _DeauthFloodData
{
  ubi_trRoot Deauthstations;
  ubi_trRootPtr DeauthstationsPtr;

  MemPool DeauthstationPool;

  u_int16_t offset_alert_flag;
  u_int16_t offset_nbr;
  u_int16_t offset_time;

  u_int32_t deauth_threshold;
  u_int32_t expire_timeout;
  u_int32_t target_limit;
  u_int32_t prune_period;

  struct timeval prune_time;

  /** Global Program Data **/
  FILE *logfile;
} DeauthFloodData;

/* this represent the deauthenticated station */
typedef struct _DeauthStation
{
  ubi_trNode Node;               /* for the splay tree */
  MemBucket *bucket;

  u_int32_t wst_addr;
  u_int8_t addr[6];
} Deauthstation;


/*  P R O T O T Y P E S
*******************************************************************************/

void SetupDeauthFlood();
void DeauthFloodInit(u_char *);
void DeauthFloodParseArgs(char *);
void DeauthFloodDetect(Packet *);
int deauthstationCompareFunc(ubi_trItemPtr, ubi_trNodePtr);
void AddDeauthStation(u_int32_t);
u_int32_t PruneDeauthstations(u_int32_t);
void DeleteDeauthstation(Deauthstation *);
void DSLog(Packet *, u_int32_t);
void DeauthFloodRestartFunction(int, void *);
void DeauthFloodCleanExitFunction(int, void *);


#endif  /* __SPP_DESFLOOD_H__ */
