/*
** Copyright (C) 2004 sebastien Gracia <sgracia@wanadoo.net>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**
** Copyright disclaimers:
** Direction Centrale des Systmes de Scurit de l'Information hereby
** disclaims all copyright interest in the changes and enhancements
** made by sebastien Gracia.
** The Directeur Central de la scurit des systmes d'information,
** Henri Serres, 5th fev 2004.  
*/


#ifndef __SPP_MACSPOOF_H__
#define __SPP_MACSPOOF_H__


/*  I N C L U D E S
*******************************************************************************/
#include "snort.h"


/*  D E F I N E S
*******************************************************************************/

#define MACSPOOF_OPT_MASKED_MAC_ADDR "masked_MAC_addr"
#define MACSPOOF_OPT_TOLERATE_GAP "tolerate_gap"
#define MACSPOOF_OPT_THRESHOLD "threshold"
#define MACSPOOF_OPT_EXPIRE_TIMEOUT "expire_timeout"
#define MACSPOOF_OPT_SPOOFED_ADDR_LIMIT "spoofed_addr_limit"
#define MACSPOOF_OPT_PRUNE_PERIOD "prune_period"

#define MACSPOOF_TOLERATE_GAP 5
#define MACSPOOF_THRESHOLD 10
#define MACSPOOF_EXPIRE_TIMEOUT 120
#define MACSPOOF_SPOOFED_ADDR_LIMIT 100
#define MACSPOOF_PRUNE_PERIOD 30


typedef struct _MACADDR
{
  u_int8_t MAC_addr[6];
  struct _MACADDR *next, *prev;
} MACADDR;


typedef struct _MaskedMACaddrList
{
  struct _MACADDR *head, *tail; 
}  MaskedMACaddrlist;


typedef struct _MACSpoofData
{
  ubi_trRoot spoofed_MACaddrs;
  ubi_trRootPtr spoofed_MACaddrsPtr;

  MemPool spoofed_MACaddrPool;

  u_int16_t offset_alert_flag;
  u_int16_t offset_seq_number;
  u_int16_t offset_gap_number;
  u_int16_t offset_time;

  u_int8_t MaskedMACaddrlist_flag;
  MaskedMACaddrlist *masked_MACaddrlist;
 
  u_int32_t tolerate_gap;
  u_int32_t threshold;
  u_int32_t expire_timeout;
  u_int32_t spoofed_addr_limit;
  u_int32_t prune_period;

  struct timeval prune_time;

  /** Global Program Data **/
  FILE *logfile;
} MACSpoofData;

/* this represent the deauthenticated station */
typedef struct _Spoofed_MACaddr
{
  ubi_trNode Node;               /* for the splay tree */
  MemBucket *bucket;

  u_int32_t wst_addr;
  u_int8_t addr[6];
} Spoofed_MACaddr;


/*  P R O T O T Y P E S
*******************************************************************************/

void SetupMACSpoof();
void MACSpoofInit(u_char *);
void MACSpoofParseArgs(char *);
void MACSpoofDetect(Packet *);
MaskedMACaddrlist *BuildMaskedMACaddrlist(char **);
MaskedMACaddrlist *AllocMaskedMACaddrlist(void);
int ParseMACAddr(u_int8_t *, char *);
MACADDR *InsertMaskedMACaddr(u_int8_t *, MaskedMACaddrlist *);
int MACSpoofedAddrCompareFunc(ubi_trItemPtr, ubi_trNodePtr);
void AddMACSpoofedAddr(u_int32_t);
u_int32_t PruneMACSpoofedAddrs(u_int32_t);
void DeleteMACSpoofedAddr(Spoofed_MACaddr *);
void MACSpoofLog(Packet *, u_int32_t); 
void MACSpoofCleanExitFunction(int, void *);
void MACSpoofRestartFunction(int, void *);


#endif  /* __SPP_MACSPOOF_H__ */
