/*
** Copyright (C) 2003 Andrew Lockhart <andrew@ingsoc.net>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* $Id: spp_rogue_ap.h,v 1.2 2003/07/27 03:21:44 andrew Exp $ */
/* Snort Preprocessor Plugin Header File RogueAp */

#include "snort.h"

#ifndef __SPP_ROGUE_AP_H__
#define __SPP_ROGUE_AP_H__

#define ROGUE_AP_NUM_ARGS 2
#define ROGUE_AP_SCAN_TIMEOUT 60
#define ROGUE_AP_EXPIRE_TIMEOUT 60 * 60 * 12 /* 12 hours */
#define ROGUE_AP_MAX_BSSIDS 255
#define ROGUE_AP_MIN_CHANNEL 1
#define ROGUE_AP_MAX_CHANNEL 11

#ifdef __SPP_ROGUE_AP_C__

typedef struct _BSSID {
  u_int8_t not_flag;
  u_int8_t bssid[6];
  time_t timestamp;
  struct _BSSID *next, *prev;
} BSSID;

typedef struct _BSSIDList {
  struct _BSSID *head, *tail;
} BSSIDList;

typedef struct _RogueApData {
  BSSIDList *pass_list;         /* list of our BSSIDs -- var ACCESS_POINTS from snort.conf */
  BSSIDList *rogue_list;        /* list of gathered rogue BSSIDs */
  u_int16_t channel_mask;       /* bitmask for channels 1-11 or 1-14 */
  u_int8_t scan_flag;           /* channel scan flag */
  time_t scan_timeout;          /* scan_timeout from snort.conf */
  time_t expire_timeout;        /* expire_timeout from snort.conf */
  time_t scan_timer;            /* timer for channel switching */
} RogueApData;

typedef enum {                  /* 802.11 frame types */
  TYPE_INVALID = -1,
  TYPE_MANAGEMENT = 0x00,
  TYPE_CONTROL = 0x01,
  TYPE_DATA = 0x02
} wifi_type_t;

typedef enum {                  /* 802.11 management frame subtypes */
  STYPE_ASSOCREQ = 0x00,		  /* Subtypes are already AND'ed with their corresponding Types */
  STYPE_ASSOCRESP = 0x01,		  /* to prevent false positives and the need for checking the type */
  STYPE_REASSOC_REQ = 0x02,	  /* independently */
  STYPE_REASSOC_RESP = 0x03,
  STYPE_PROBEREQ = 0x04,
  STYPE_PROBERESP = 0x05,
  STYPE_BEACON = 0x08,
  STYPE_ATIM = 0x09,
  STYPE_DISASSOC = 0x0a,
  STYPE_AUTH = 0x0b,
  STYPE_DEAUTH = 0x0c,

  STYPE_INVALID = 0x0d,
/*   STYPE_INVALID = 0x0e, */
/*   STYPE_INVALID = 0x0f, */
/*   STYPE_INVALID = 0x10, */
/*   STYPE_INVALID = 0x11, */
/*   STYPE_INVALID = 0x12, */
/*   STYPE_INVALID = 0x13, */
/*   STYPE_INVALID = 0x14, */
/*   STYPE_INVALID = 0x15, */
/*   STYPE_INVALID = 0x16, */
/*   STYPE_INVALID = 0x17, */
/*   STYPE_INVALID = 0x18, */
/*   STYPE_INVALID = 0x19, */


/* 802.11 control frame subtypes */
  STYPE_PS = 0x0a,
  STYPE_RTS = 0x0b,
  STYPE_CTS =  0x0c,
  STYPE_ACK = 0x0d,
  STYPE_CFEND = 0x0e,
  STYPE_CFEND_CFACK = 0x0f,

/* 802.11 data frame subtypes */
  STYPE_DATA = 0x00,
  STYPE_CFACK = 0x01,
  STYPE_CFPOLL = 0x02,
  STYPE_CFACK_CFPOLL = 0x03,
  STYPE_NULLFUNC = 0x04,
  STYPE_CFACK_NULLFUNC = 0x05,
  STYPE_CFPOLL_NULLFUNC = 0x06,
  STYPE_CFACK_CFPOLL_NULLFUNC = 0x07
} wifi_stype_t;


BSSIDList *BuildApList(char **);
u_int16_t BuildChannelMask(char **);
BSSIDList *AllocBSSIDList(void);
void DestroyBSSIDList(BSSIDList *);
BSSID *InsertBSSID(u_int8_t *, BSSIDList *);
BSSID *RemoveBSSID(BSSID *, BSSIDList *);
int ParseBSSID(u_int8_t *, char *);

#endif /* __SPP_ROGUE_AP_C__ */


void SetupRogueAp();
void RogueApInit(u_char *);
void RogueApParseArgs(char *);
void RogueApDetect(Packet *);
void RogueApRestart(int);
void RogueApCleanExit(int);



#endif  /* !__SPP_ROGUE_AP_H__ */
